import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:schedule_badge_updates/schedule_badge_updates.dart';

void main() => runApp(MyApp());

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  String _badgeSupported = 'Not set';

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String badgeSupported;
    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      ScheduleBadgeUpdates.setBadge(16);
      String res = await ScheduleBadgeUpdates.scheduleBadge(15, DateTime.now().add(Duration(seconds: 15)));
      print(res);
      bool isBadge = await ScheduleBadgeUpdates.isBadgeSupported();
      if (!isBadge) {
        badgeSupported = 'Not supported';
      }
      else {
        badgeSupported = 'YASS!!!';
      }
    } on PlatformException {
      badgeSupported = 'Failed to get platform version.';
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _badgeSupported = badgeSupported;
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Plugin example app'),
        ),
        body: Center(
          child: Text('Running on: $_badgeSupported\n'),
        ),
      ),
    );
  }
}
