import Flutter
import UIKit

public class SwiftScheduleBadgeUpdatesPlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "schedule_badge_updates", binaryMessenger: registrar.messenger())
    let instance = SwiftScheduleBadgeUpdatesPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        // https://stackoverflow.com/questions/28011118/add-badge-to-app-icon-in-ios-8-with-swift
    let application = UIApplication.shared
    
    if #available(iOS 10.0, *) {
      let notificationCenter = UNUserNotificationCenter.current()
      notificationCenter.requestAuthorization(options: .badge)
        { (granted, error) in }
    }
    else {
      application.registerUserNotificationSettings(UIUserNotificationSettings(types: [.badge], categories: nil))
    }

    application.registerForRemoteNotifications()
      

    if (call.method == "scheduleBadge") {
      guard let args = call.arguments as? [String: Any] else {
        return result("Missing params")
      }

      if #available(iOS 10.0, *) {
        let notificationCenter = UNUserNotificationCenter.current()
        
        let badgeCount = args["badge"] as! NSNumber
        let content = UNMutableNotificationContent()
        content.badge = badgeCount

        let date = args["date"] as! [String: Int]
        var dateComponents = DateComponents()
        dateComponents.calendar = Calendar.current // Want to get current timezone
        dateComponents.year = date["year"]
        dateComponents.month = date["month"]
        dateComponents.day = date["day"]
        dateComponents.hour = date["hour"]
        dateComponents.minute = date["minute"]
        dateComponents.second = date["second"]

        let trigger = UNCalendarNotificationTrigger(
          dateMatching: dateComponents, repeats: true)

        let uuidString = UUID().uuidString

        let request = UNNotificationRequest(
          identifier: uuidString, 
          content: content,
          trigger: trigger
        )

        notificationCenter.add(request) { (error) in
          if error != nil {
            return result("Failed to schedule badge update")
          }
        }
        return result(uuidString)
      }
      else {
        // TODO: Implement for iOS 8 https://shrikar.com/ios-8-notifications-in-swift/
        // Maybe not, it is totally different and we are already three years ahead...
        return result("TODO: Implement for iOS8")
      }
    }
    else if (call.method == "clearBadge") {
      application.applicationIconBadgeNumber = 0
    }
    else if (call.method == "setBadge") {
      // https://stackoverflow.com/questions/53898765/flutter-plugin-invoking-ios-and-android-method-including-parameters-not-working
      guard let badgeCount = call.arguments as? Int else {
        return result("Missing badge number")
      }
      application.applicationIconBadgeNumber = badgeCount
    }
    else if (call.method == "isBadgeSupported") {
      /*
      if #available(iOS 10.0, *) {
        var notificationSettings: UNNotificationSettings?
        let semasphore = DispatchSemaphore(value: 0)

        DispatchQueue.global().async {    
            UNUserNotificationCenter.current().getNotificationSettings { setttings in
                notificationSettings = setttings
                semasphore.signal()
            }
        }

        semasphore.wait()
        guard let authorizationStatus = notificationSettings?.authorizationStatus else { return result(false) }
        //return result(authorizationStatus)
        return result(true)
      }
      else {
        return result("TODO: Implement for iOS8")
      }
      */
      return result(true)
    }
    else {
      result("Unknown method call '\(call.method)'")
    }
  }
}
